@@ -140,6 +140,14 @@ public class BookStore {
         return books.containsKey(id);
     }
     
+    @POST
+    @Path("books/check2")
+    @Produces("text/plain")
+    @Consumes("text/plain")
+    public Boolean checkBook2(Long id) {
+        return books.containsKey(id);
+    }
+    
     
     @GET
     @Path("timetable")
@@ -395,6 +403,12 @@ public class BookStore {
         return new BookInfo(doGetBook("123"));
     }
     
+    @GET
+    @Path("/books/interface/adapter")
+    public BookInfoInterface getBookAdapter2() throws Exception {
+        return new BookInfo2(doGetBook("123"));
+    }
+    
     @PathParam("bookId")
     public void setBookId(String id) {
         currentBookId = id;
@@ -500,7 +514,7 @@ public class BookStore {
     
     @POST
     @Path("/books/customstatus")
-    @Produces("text/xml")
+    @Produces("application/xml")
     @Consumes("text/xml")
     public Book addBookCustomFailure(Book book, @Context HttpServletResponse response) {
         response.setStatus(333);
@@ -625,6 +639,27 @@ public class BookStore {
         return new Long(theBookId);
     }
     
+    @POST
+    @Path("/booksecho")
+    @Consumes("text/plain")
+    @Produces("text/plain")
+    public Response echoBookNameAndHeader(@HeaderParam("CustomHeader") String headerValue, String name) {
+        return Response.ok().entity(name).header("CustomHeader", headerValue).build();
+    }
+    
+    @Path("/bookstoresub")
+    public BookStore echoThroughBookStoreSub() {
+        return this;
+    }
+    
+    @POST
+    @Path("/booksecho2")
+    @Consumes("text/plain")
+    @Produces("text/plain")
+    public Response echoBookNameAndHeader2(String name) {
+        return echoBookNameAndHeader(httpHeaders.getRequestHeader("CustomHeader").get(0), name);
+    }
+    
     @GET
     @Path("/cd/{CDId}/")
     public CD getCD() {
@@ -671,6 +706,13 @@ public class BookStore {
         cds.put(cd1.getId(), cd1);
     }
     
+    @XmlJavaTypeAdapter(BookInfoAdapter.class)
+    private static interface BookInfoInterface {
+        String getName();
+        
+        long getId();
+    }
+    
     static class BookInfo {
         private String name;
         private long id;
@@ -703,6 +745,16 @@ public class BookStore {
         }
     }
     
+    static class BookInfo2 extends BookInfo implements BookInfoInterface {
+        public BookInfo2() {
+            
+        }
+        
+        public BookInfo2(Book b) {
+            super(b);
+        }
+    }
+    
     public static class BookInfoAdapter extends XmlAdapter<Book, BookInfo> {
 
         @Override
